#define BTN1        12		 // кнопка подключена сюда (PIN --- КНОПКА --- GND)
#define BTN2        15       // кнопка подключена сюда (PIN --- КНОПКА --- GND)
#define BUZZER      16       // Пин базера пассивного
#define TM1637_CLK  13
#define TM1637_DIO   2
#define RELAY       14      // Пин управление реле включения духовки

#include <ESP8266WiFi.h>
#include <ArduinoOTA.h>
#include <PubSubClient.h>
#include <Ticker.h>
#include "TM1637.h"
#include <WiFiUdp.h>
#include "RTClib.h"
#include "TimeLib.h"

TM1637 tm1637(TM1637_CLK,TM1637_DIO);
Ticker blinker;
WiFiClient ESPDclient;
PubSubClient MQTTclient(ESPDclient);

const char* ssid = "********";
const char* password = "********";

static const char ntpServerName[] = "ua.pool.ntp.org";
const int timeZone = 3;     // Kiev Time
WiFiUDP Udp;
unsigned int localPort = 8888;  // local port to listen for UDP packets
time_t getNtpTime();

RTC_DS3231 rtc;

byte lastSecond;
int secFr = 0;
bool flash = 0;
byte key = 0;
int keySpeed;
int timmer = 0, lastTimmer = 0, mqttTimmer = 0;
int lastSecTim = 0;
byte h = 0;
byte m = 0;
byte s = 0;
bool tpause = true;      // Флаг выкл реле духовки
bool statusWifi = false;

byte tWait = 0, tempKey = 0;

byte _hour=21, _minute=55, _second=1, _month=8, _day=28, lastDay=28, _dayOfWeek=4;
int _year=2018;

// ----------змінні для роботи з mqtt сервером
char mqtt_server[21] = "********";
int  mqtt_port = 1883;
char mqtt_user[25] = "********";
char mqtt_pass[25] = "********";
char mqtt_name[25] = "";
//char mqtt_pub_temp[25] = "********/temp";
char mqtt_pub_timer[25] = "duh_timer/timer";
char mqtt_pub_pause[25] = "duh_timer/pause";

void setup() {
//  Serial.begin(9600);

  pinMode(BTN1, INPUT);
  pinMode(BTN2, INPUT);
  pinMode(BUZZER, OUTPUT);
  digitalWrite(BUZZER, LOW);
  pinMode(RELAY, OUTPUT);
  digitalWrite(RELAY, !tpause);
  
  tm1637.init();
  tm1637.set(7);
  
  blinker.attach(1, updateLocalTime);
  
  wifiConnect();
  
  Udp.begin(localPort);
  setSyncProvider(getNtpTime);
  setSyncInterval(300);

  if (!rtc.begin()) {
//    Serial.println("Couldn't find RTC");
    while (1);
  }
   if (rtc.lostPower()) {
//    Serial.println("RTC lost power!");
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
  }

  updateTime();
  displayTime();
  
  MQTTclient.setServer(mqtt_server, mqtt_port);

  ArduinoOTA.setHostname("Duhovka_Timer");
  ArduinoOTA.onStart([]() {
//    Serial.println("Start");  //  "Начало OTA-апдейта"
  });
  ArduinoOTA.onEnd([]() {
//    Serial.println("\nEnd");  //  "Завершение OTA-апдейта"
  });
  ArduinoOTA.onProgress([](unsigned int progress, unsigned int total) {
//    Serial.printf("Progress: %u%%\r", (progress / (total / 100)));
  });
  ArduinoOTA.onError([](ota_error_t error) {
//    Serial.printf("Error[%u]: ", error);
//    if (error == OTA_AUTH_ERROR) Serial.println("Auth Failed");
    //  "Ошибка при аутентификации"
//    else if (error == OTA_BEGIN_ERROR) Serial.println("Begin Failed");
    //  "Ошибка при начале OTA-апдейта"
//    else if (error == OTA_CONNECT_ERROR) Serial.println("Connect Failed");
    //  "Ошибка при подключении"
//    else if (error == OTA_RECEIVE_ERROR) Serial.println("Receive Failed");
    //  "Ошибка при получении данных"
//    else if (error == OTA_END_ERROR) Serial.println("End Failed");
    //  "Ошибка при завершении OTA-апдейта"
  });
  ArduinoOTA.begin();
}
//---------------------------------------
void updateMQTT(){
  if(!MQTTclient.connected()) {
    reconnect();
  }
  if(MQTTclient.connected()) {
//    MQTTclient.publish(mqtt_pub_temp, (String(tempSensors)).c_str());
    MQTTclient.publish(mqtt_pub_timer, (String(timmer)).c_str());
    MQTTclient.publish(mqtt_pub_pause, (String(tpause)).c_str());
    mqttTimmer = timmer;
  }
}
//---------------------------------------

void updateTime() {
  if(WiFi.status() != WL_CONNECTED) {
//    Serial.println("Connection internet - failed!!!");
    DateTime now = rtc.now();
    _year = now.year();
    _month = now.month();
    _day = now.day();
    _hour = now.hour();
    _minute = now.minute();
    _second = now.second();
    return;
  }
  _hour = hour();
  _minute = minute();
  _second = second();
  _dayOfWeek = weekday();
  _year = year();
  _month = month();
  _day = day();
//  Serial.println(String(_day) + "." + String(_month) + "." + String(_year) + " " + String(_dayOfWeek) + " " + String(_hour) + ":" + String(_minute) + ":" + String(_second));
  rtc.adjust(DateTime(_year, _month, _day, _hour, _minute, _second));
}//---------------------------------------
void wifiConnect() {
  statusWifi = 0;
//WiFi.disconnect();
  WiFi.mode(WIFI_STA);
//  Serial.println();
//  Serial.printf("Connecting to %s ", ssid);
  WiFi.begin(ssid, password);
  for(int i = 0; i < 20; i++){
    if(WiFi.status() == WL_CONNECTED){
//      Serial.println(" connected");
      statusWifi = 1;
      return;
    }
    delay(500);
//    Serial.print(".");
  }
//  Serial.println("NOT connected");
  ESP.restart();
}//---------------------------------------
void reconnect() { 
  if(!ESPDclient.connected() && WiFi.status() == WL_CONNECTED) {
//    Serial.print("MQTT reconnection...");
    if(MQTTclient.connect(mqtt_name, mqtt_user, mqtt_pass)) {
//      Serial.println("connected");
    } else {
//      Serial.print("failed, rc = ");
//      Serial.println(MQTTclient.state());
    }
  }
}
//=======================================
void alarm() {
  digitalWrite(RELAY, LOW);
  tpause = 1; 
  updateMQTT();
  for(int i = 0; i < 120; i++){  //2 min bips
    tm1637.clearDisplay();
    tm1637.point(1);
    tm1637.display(0, 0);
    tm1637.display(1, 0);
    tm1637.display(2, 0);
    tm1637.display(3, 0);
    bip();
    bip();
    bip();
    tm1637.clearDisplay();
    delay(280);
    if(digitalRead(BTN1) == HIGH && digitalRead(BTN2) == HIGH) {
      bip();
      delay(500);
      return;
    }
  }
}
//--------------------------------------
void updateLocalTime() {
  _second++;
  if(_second>59) {
    _second = 0;
    _minute++;
    if(_minute>59) {
      _minute = 0;
      _hour++;
      if(_hour>23) {
        _hour = 0;
        _day++;
        if(_day==32 || (_day==31 && (_month==4 || _month==6 || _month==9 || _month==11)) || (_month==2 && ((_day==29 && _year%4!=0) || (_day==30 && _year%4==0)))) {
          _day=1;
          _month++;
          if(_month>12){
            _month=1;
            _year++;
          }
        }
      }
    }
  } 
}//---------------------------------------
void bip(){
  tone(BUZZER, 2500);
  delay(keySpeed < 8 ? 120 : keySpeed < 40 ? 60 : 20);
  noTone(BUZZER);
  delay(keySpeed < 8 ? 120 : keySpeed < 40 ? 60 : 20);
}//---------------------------------------
void displayTime(){
  tm1637.point(flash);
  tm1637.display(0, _hour / 10);
  tm1637.display(1, _hour % 10);
  tm1637.display(2, _minute / 10);
  tm1637.display(3, _minute % 10);
//  Serial.println(String(_hour) + ":" + String(_minute));
}//-------------------------------------
void displayTimer(){
  h = (int)timmer / 3600;
  m = (int)timmer / 60 - h * 60;
  s = timmer - m * 60;
  tm1637.point(1);
  if(timmer > 3600) {
    if(h > 9) {
      tm1637.display(0, h / 10);
    }
    tm1637.display(1, h % 10);
    tm1637.display(2, m / 10);
    tm1637.display(3, m % 10);
//    Serial.println(String(h) + ":" + String(m));
  } else {
    if(m > 9) {
      tm1637.display(0, m / 10);
    }
    if(m > 0) {
      tm1637.display(1, m % 10);
    }
    if(s > 9 || m > 0) {
      tm1637.display(2, s / 10);
    }
    tm1637.display(3, s % 10);
//    Serial.println(String(m) + ":" + String(s));
  }
}//-------------------------------------
void klav() {
  tempKey = 0;
  if(digitalRead(BTN1) == HIGH && digitalRead(BTN2) == LOW) tempKey = 1;
  if(digitalRead(BTN1) == LOW && digitalRead(BTN2) == HIGH) tempKey = 2;
  if(digitalRead(BTN1) == HIGH && digitalRead(BTN2) == HIGH) key = 0;
//  if(tempKey != 0) Serial.println(tempKey);
  if(tempKey != 0 && tWait < 10 && key != 0) {
    delay(100);
    tWait++;
  } else { // все остальное
  
    if(key != 0 && (keySpeed == 0 || (h == 1 && m == 0) || (h == 0 && m == 59) || (m == 9 && s == 00))) tm1637.clearDisplay();

    if(key != 0) {
      keySpeed++;
    } else {
      keySpeed = 0;
      tWait = 0;
    }
//------------------------------------- +1  
    if(key == 1) {
      timmer = (int(timmer / 60) + 1) * 60;
      if(timmer > 10800) timmer = 10800; // ограничение таймера
      displayTimer();
    }
//------------------------------------- -1
    if(key == 2) {
      lastTimmer = timmer;
      timmer = (int(timmer / 60) - (timmer % 60 == 0 ? 1 : 0)) * 60;
      if(timmer < 0) timmer = 0;
      if(timmer != 0) {
        displayTimer();
      } else if(lastTimmer !=0){
        if (tpause){
          displayTime();
        }else{
          alarm();
        }
      }
    }
//------------------------------------- звук клавиш  
    if(key != 0) {
      bip();
    }
  }
  key = tempKey;
}
//--------------------------------------- период 20мс = 5 раз в сек.
void menu(){
  klav();
//------------------------------------------------------  работа таймера  
  if(timmer != 0 && key == 0 && tpause == 0) {
    if(lastSecTim != _second) {
      lastSecTim = _second;
      timmer--;
      if(!timmer) {
//        Serial.println("STOP timer"); 
        alarm();
      }
    }
  }
//----------------------------------------------------- Пауза в таймере = сигнал 5 бипов каждые 10 секунд  
  if(timmer != 0 && tpause && _second % 10 == 0 && key == 0) {
    bip();
  }
//----------------------------------------------------- Кнопка Старт/Пауза
  if(digitalRead(BTN1) == HIGH && digitalRead(BTN2) == HIGH && key == 0){
    if(timmer != 0) {
      tpause = !tpause;
      digitalWrite(RELAY, !tpause);
    }
    updateMQTT();
    bip();
    delay(500);
  }
}//---------------------------------------

void loop() {
  ArduinoOTA.handle();

//----------------------------  цикл = 1сек
  if(_second != lastSecond) {
    lastSecond = _second;
    secFr = 0;        //начало каждой секунды
  } else secFr++;

//--------------------------  мигание чаще, если нет WiFi
  if(millis() % (statusWifi ? 1000 : 500) < (statusWifi ? 500 : 400) && flash || key != 0) {
    flash = 0;
    if(timmer == 0) {
      displayTime();
    }
    else {
      displayTimer();
    }
  }
  else if(millis() % (statusWifi ? 1000 : 500) > (statusWifi ? 500 : 400) && !flash || key != 0) {
    flash = 1;
    if(timmer == 0) {
      displayTime();
    }
    else {
      tm1637.clearDisplay();
    }
  }

//----------------------------------------------  Обновляем время один раз в сутки на 5й секунде
  if(secFr==0 && _hour == 0 && _minute== 5 && _second == 0) {
    updateTime();
  }

//----------------------------------------------  каждый час сигнал
  if(secFr==0 && _second==0 && _minute==0 && _hour < 23 && _hour > 6) {
    bip();
    bip();
  }
    
  menu();
  
//--------------------------------------------------------  проверка подключения WiFi каждые 10 минут на 5й секунде
  if(secFr == 0 && _second == 5 && _minute % 10 == 1 && WiFi.status() != WL_CONNECTED) wifiConnect();
  
//--------------------------------------------------------  MQTT кажд 3s если таймер включен
  if(secFr == 0 && _second % 3 == 0 && timmer != mqttTimmer){
    updateMQTT();
  }
}
